"""
XGngeo: a frontend for Gngeo in GTK ^^.
Copyleft 2003 Choplair-network

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""
from string import split
import os.path

class Configfile:
	"""XGngeo manages two configuration files:
- Gngeo's configuration file (`gngeorc') to modify emulator paramas (graphic, path, keys, etc.);
- his own configuration file (`xgngeo.conf') to customize some features (history, preview images, etc.).
XGngeo's configuration file uses the same syntax than Gngeo's."""

	def __init__(self,gngeo,xgngeo):
		self.gngeoPath = gngeo #Path of Gngeo's config file.
		self.xgngeoPath = xgngeo #Path of XGngeo's config file.
		
	def exists(self):
		return os.path.isfile(self.gngeoPath),os.path.isfile(self.xgngeoPath)

	def getParams(self):
		dict = [{},{}]
		
		#Gngeo config file (which must exists!)
		file = open(self.gngeoPath,"r") #Open
		content = file.readlines() #Read		
		file.close() #And close :p
		for line in content:
			line.strip()
			if line[0]!="#" and line!="\n":
				key,val = split(line)
				dict[0][key.strip()] = val.strip()

		#XGngeo config file (which is optional)
		if self.exists()[1]:
			file = open(self.xgngeoPath,"r") #Open
			content = file.readlines() #Read		
			file.close() #And close :p
			for line in content:
				line.strip()
				if line[0]!="#" and line!="\n":
					key,val = split(line)
					dict[1][key.strip()] = val.strip()

		return dict

	def write(self,gngeoDict,xgngeoDict,version):
		#Top comments :p	
		gngeoContent = "# Gngeo configuration file.\n\
# Generated by XGngeo version %i (http://choplair.tuxfamily.org/) ^o^\n\
\n" % version
		xgngeoContent = "# XGngeo configuration file.\n\
# This file is dedicated to customize XGngeo's own features. ^o^\n\
# It uses same syntax as Gngeo's.\n\
\n"

		#WAGLAMOT (`Write A Gngeorc Like A Maman Ours Technology') version 2 in action!
		i=0
		for dict in gngeoDict,xgngeoDict:
			for key, val in dict.items():
				if i==0: gngeoContent += "%s %s\n" % (key,val)
				else: xgngeoContent += "%s %s\n" % (key,val)
			i+=1

		#Add blank line at the end
		gngeoContent += "\n"
		xgngeoContent += "\n"

		file = open(self.gngeoPath,"w") #Open (create if doesn't exist)
		file.write(gngeoContent) #Write		
		file.close() #And close
		del gngeoContent

		file = open(self.xgngeoPath,"w") #Open (create if doesn't exist)
		file.write(xgngeoContent) #Write		
		file.close() #And close
		del xgngeoContent
